'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var lezerProcessfulText = require('lezer-processful-text');
var langCss = require('@codemirror/lang-css');
var langJavascript = require('@codemirror/lang-javascript');
var language = require('@codemirror/language');
var highlight = require('@codemirror/highlight');

/**
A language provider based on the [Lezer HTML
parser](https://github.com/lezer-parser/html), wired up with the
JavaScript and CSS parsers to parse the content of `<script>` and
`<style>` tags.
*/
const textLanguage = language.LezerLanguage.define({
    parser: lezerProcessfulText.parser.configure({
        props: [
            language.indentNodeProp.add({
                Element(context) {
                    let after = /^(\s*)(<\/)?/.exec(context.textAfter);
                    if (context.node.to <= context.pos + after[0].length)
                        return context.continue();
                    return context.lineIndent(context.state.doc.lineAt(context.node.from)) + (after[2] ? 0 : context.unit);
                },
                "OpenTag CloseTag SelfClosingTag"(context) {
                    return context.column(context.node.from) + context.unit;
                },
                Document(context) {
                    if (context.pos + /\s*/.exec(context.textAfter)[0].length < context.node.to)
                        return context.continue();
                    let endElt = null, close;
                    for (let cur = context.node;;) {
                        let last = cur.lastChild;
                        if (!last || last.name != "Element" || last.to != cur.to)
                            break;
                        endElt = cur = last;
                    }
                    if (endElt && !((close = endElt.lastChild) && (close.name == "CloseTag" || close.name == "SelfClosingTag")))
                        return context.lineIndent(context.state.doc.lineAt(endElt.from)) + context.unit;
                    return null;
                }
            }),
            language.foldNodeProp.add({
                Element(node) {
                    let first = node.firstChild, last = node.lastChild;
                    if (!first || first.name != "OpenTag")
                        return null;
                    return { from: first.to, to: last.name == "CloseTag" ? last.from : node.to };
                }
            }),
            highlight.styleTags({
                AttributeValue: highlight.tags.string,
                "Text RawText": highlight.tags.content,
                "StartTag StartCloseTag SelfCloserEndTag EndTag SelfCloseEndTag": highlight.tags.angleBracket,
                TagName: highlight.tags.tagName,
                "MismatchedCloseTag/TagName": [highlight.tags.tagName, highlight.tags.invalid],
                AttributeName: highlight.tags.propertyName,
                UnquotedAttributeValue: highlight.tags.string,
                Is: highlight.tags.definitionOperator,
                "EntityReference CharacterReference": highlight.tags.character,
                Comment: highlight.tags.blockComment,
                ProcessingInst: highlight.tags.processingInstruction,
                DoctypeDecl: highlight.tags.documentMeta
            })
        ],
        nested: lezerProcessfulText.configureNesting([
            { tag: "script",
                attrs(attrs) {
                    return !attrs.type || /^(?:text|application)\/(?:x-)?(?:java|ecma)script$|^module$|^$/i.test(attrs.type);
                },
                parser: langJavascript.javascriptLanguage.parser },
            { tag: "style",
                attrs(attrs) {
                    return (!attrs.lang || attrs.lang == "css") && (!attrs.type || /^(text\/)?(x-)?(stylesheet|css)$/i.test(attrs.type));
                },
                parser: langCss.cssLanguage.parser }
        ])
    }),
    languageData: {
        commentTokens: { block: { open: "<!--", close: "-->" } },
        indentOnInput: /^\s*<\/$/
    }
});
/**
Language support for HTML, including
[`htmlCompletion`](https://codemirror.net/6/docs/ref/#lang-html.htmlCompletion) and JavaScript and
CSS support extensions.
*/
function text() {
    return new language.LanguageSupport(textLanguage, []);
}

exports.text = text;
exports.textLanguage = textLanguage;
